@@ -1,6 +1,6 @@
 from mlagents_envs.environment import UnityEnvironment
 from mlagents_envs.side_channel.engine_configuration_channel import EngineConfigurationChannel
-from mlagents_envs.base_env import BaseEnv
+from mlagents_envs.base_env import BaseEnv, ActionTuple
 from gym_unity.envs import UnityToGymWrapper
 from PIL import Image, ImageDraw
 import gym
@@ -9,6 +9,8 @@ import numpy as np
 import cv2 as cv
 import io
 from threading import Thread
+import keyboard
+import time
 
 num_vector = 7
 
@@ -31,7 +33,6 @@ class CogEnvDecoder:
         self.allow_multiple_obs = allow_multiple_obs
         self.spec = self._env.spec
 
-        print("init")
 
     @property
     def observation_space(self):
@@ -63,8 +64,23 @@ class CogEnvDecoder:
         final_obs = [img, self_pos, enemy_pos, self_info, goal1, goal2, goal3, goal4, goal5, collision_info]
         return final_obs
 
+    def action_transfer(self, action):
+        action = np.array(action).reshape((1, 4))
+        action_tuple = ActionTuple()
+        action_tuple.add_continuous(action)
+        self._env._env.set_actions(self._env.name, action_tuple)
+        decision_step, terminal_step = self._env._env.get_steps(self._env.name)
+        self._env._check_agents(max(len(decision_step), len(terminal_step)))
+        #print(self._env.name)
+        if len(terminal_step) != 0:
+            # The agent is done
+            self._env.game_over = True
+            return self._env._single_step(terminal_step)
+        else:
+            return self._env._single_step(decision_step)
+
     def step_action(self, action):
-        obs, reward, done, info = self._env.step(action)
+        obs, reward, done, info = self.action_transfer(action)
         # obs = self._get_obs(obs)
         img = self._GetImg(obs)
         self_pos = self._GetSelfPos(obs)  # Blue one Position
@@ -153,7 +169,7 @@ class CogEnvDecoder:
             opencv_img = opencv_img[:, :, ::-1].copy()
             return opencv_img
         else:
-            print('The Environment has not finished loading')
+            #print('The Environment has not finished loading')
             gray0 = np.zeros((300, 300), dtype=np.uint8)
             return gray0
 
@@ -242,10 +258,23 @@ class CogEnvDecoder:
         return [coll_time, cont_coll_time]
 
     def _thread_step(self):
+        quit = False
+        #print('1')
         while True:
             self._env._env.step()
+            time.sleep(0.015)
+            if keyboard.is_pressed('q'):
+                quit = True
+                self.close()
+                self._thread.join()
+
+
 
     def step_thread_begin(self):
-        thread = Thread(target=self._thread_step())
-        thread.start()
-        thread.join()
\ No newline at end of file
+        self._thread = Thread(target=self._thread_step)
+        #print('2')
+        self._thread.start()
+        #print('3')
+        #self._thread.join()
+        #print('4')
+
